/* eslint-disable no-magic-numbers */
import { OPTIONS } from '@/constants/map';
import { currentBackgroundImagePathSelector } from '@/redux/backgrounds/background.selectors';
import { mapDataPositionSelector, mapDataSizeSelector } from '@/redux/map/map.selectors';
import { projectDataSelector } from '@/redux/project/project.selectors';
import { Point } from '@/types/map';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import { View } from 'ol';
import BaseLayer from 'ol/layer/Base';
import TileLayer from 'ol/layer/Tile';
import { XYZ } from 'ol/source';
import { useMemo } from 'react';
import { useSelector } from 'react-redux';
import { getMapTileUrl } from './getMapTileUrl';

interface UseOlMapConfigResult {
  view: View;
  layers: BaseLayer[];
}

export const useOlMapConfig = (): UseOlMapConfigResult => {
  const mapPosition = useSelector(mapDataPositionSelector);
  const mapSize = useSelector(mapDataSizeSelector);
  const currentBackgroundImagePath = useSelector(currentBackgroundImagePathSelector);
  const project = useSelector(projectDataSelector);
  const pointToProjection = usePointToProjection();

  const center = useMemo(() => {
    const centerPoint: Point = {
      x: mapPosition.x,
      y: mapPosition.y,
    };

    return pointToProjection(centerPoint);
  }, [mapPosition, pointToProjection]);

  const view = useMemo(
    () =>
      new View({
        center,
        zoom: mapPosition.z,
        showFullExtent: OPTIONS.showFullExtent,
      }),
    [center, mapPosition],
  );

  const tileLayer = useMemo(
    (): TileLayer<XYZ> =>
      new TileLayer({
        visible: true,
        source: new XYZ({
          url: getMapTileUrl({ projectDirectory: project?.directory, currentBackgroundImagePath }),
          maxZoom: mapSize.maxZoom,
          minZoom: mapSize.minZoom,
          tileSize: mapSize.tileSize,
          wrapX: OPTIONS.wrapXInTileLayer,
        }),
      }),
    [mapSize, currentBackgroundImagePath, project?.directory],
  );

  return {
    view,
    layers: [tileLayer],
  };
};
